import zope.interface
from urllib2 import URLError

from AccessControl import ClassSecurityInfo
from persistent.list import PersistentList

from Products.Archetypes.utils import shasattr
try:
    from Products.LinguaPlone.public import *
except ImportError:
    # No multilingual support
    from Products.Archetypes.public import *

from interfaces import IRSSFeedRecipe
try:
    import feedparser
except:
    from Products.PloneRSSPortlet import feedparser

RSSFeedRecipeSchema =  BaseSchema.copy() + Schema((

    StringField(
        name='url',
        languageIndependent=False,
        widget=StringWidget(
            size=70,
            label='Home URL',
            label_msgid='label_feed_home_url',
            description='The address for homepage of the feed.',
            description_msgid='help_feed_home_url',
            i18n_domain='plone',
        ),
    ),

    StringField(
        name='feedURL',
        languageIndependent=False,
        widget=StringWidget(
            size=70,
            label='Feed URL',
            label_msgid='label_feed_url',
            description='The source address for the feed.',
            description_msgid='help_feed_url',
            i18n_domain='plone',
        ),
    ),

    IntegerField(
        name='entriesSize',
        default=5,
        languageIndependent=True,
        widget=IntegerWidget(
            label='Entries Size',
            label_msgid='label_size_feed',
            description='The number of items from the feed to display.',
            description_msgid='help_size_feed',
            i18n_domain='plone',
        ),
    ),

    IntegerField(
        name='entriesWithDescription',
        default=1,
        languageIndependent=True,
        widget=IntegerWidget(
            label='Entries with description',
            label_msgid='label_with_desc',
            description='The number of items from the feed to display with description.',
            description_msgid='help_with_desc',
            i18n_domain='plone',
        ),
    ),

    IntegerField(
        name='entriesWithThumbnail',
        default=1,
        languageIndependent=True,
        widget=IntegerWidget(
            label='Entries with thumbnail',
            label_msgid='label_with_thumb',
            description='The number of items from the feed to display with a thumbnail/image.',
            description_msgid='help_with_thumb',
            i18n_domain='plone',
        ),
    ),


    StringField(
        name='target',
        languageIndependent=True,
        widget=StringWidget(
            label='Link Target',
            label_msgid='label_link_target',
            description='The value to be used as the link target attribute.',
            description_msgid='help_link_target',
            i18n_domain='plone',
        ),
    ),

))

class RSSFeedRecipe(BaseContent):
    """This content type allows you to show external feeds in your portal."""

    zope.interface.implements(IRSSFeedRecipe)
    
    schema = RSSFeedRecipeSchema
    security = ClassSecurityInfo()

    meta_type = 'RSSFeedRecipe'
    archetype_name = 'RSS Feed Recipe'
    content_icon = 'RSSFeedRecipe_icon.png'
    
    _at_rename_after_creation = True

    def reindexObject(self, **kw):
        BaseContent.reindexObject(self, **kw)
        if getattr(self, '_v_updateFeed', False):
            self.updateFeed()
        self._v_updateFeed = False
        
    def setFeedURL(self, value, **kwargs):
        """Sets the feed URL."""
        field = self.getField('feedURL')
        old_value = field.get(self, **kwargs)
        if value != old_value:
            self.getField('feedURL').set(self, value, **kwargs)
            self._v_updateFeed = True
            self._v_updatedUrl = True

    def setEntriesSize(self, value, **kwargs):
        """Sets the number of feed entries."""
        field = self.getField('entriesSize')
        old_value = field.get(self, **kwargs)
        if int(value) != int(old_value):
            self.getField('entriesSize').set(self, value, **kwargs)
            self._v_updateFeed = True

    security.declarePrivate('getCacheKeywords')
    def getCacheKeywords(self):
        """Returns the keywords to take into account when caching."""
        kw = {
            'feed': self.getFeedURL(),
        }
        return kw

    security.declarePrivate('getAttributesMapping')
    def getAttributesMapping(self):
        """Returns the attributes mapping."""
        mapping = {
           'Title': 'title',
           'Description': 'summary',
           'getURL': 'link',
           'Date': 'modified',
           'Category': 'category',
        }
        return mapping


    def updateFeed(self):
        result = PersistentList()
        rssportlet = self
        feed_url = rssportlet.getFeedURL()
        feed = feedparser.parse(feed_url)

        # don't remove all feeds if the server is temporarily unavailable
        # only if we changed the feed url and the server is unavailable
        # we should do that
        if getattr(self, '_v_updatedUrl', False):
            self.savedFeed = PersistentList()
            self._v_updatedUrl = False
        bozo_exception = feed.get('bozo_exception')
        if bozo_exception and isinstance(bozo_exception, URLError):
            if not hasattr(self, 'savedFeed'):
                self.savedFeed = PersistentList()
            return self.savedFeed

        # XXX Write proper ETAG cache logic
        # self._v_etag = feed.etag
        size = rssportlet.getEntriesSize()
        if len(feed.entries) < size:
            size = len(feed.entries)

        for x in range(size):
            item = {}
            item.update(feed.entries[x])
            mapping = rssportlet.getAttributesMapping()

            for k, v in mapping.items():
                if shasattr(feed.entries[x], v):
                    item[k] = getattr(feed.entries[x], v)
                else:
                    item[k] = None

            result.append(item)

        self.savedFeed = result
        return result

    def getFeed(self):
        """ """
        if hasattr(self, 'savedFeed'):
            return self.savedFeed
        
        return self.updateFeed()
        

registerType(RSSFeedRecipe)
